@@ -35,7 +35,6 @@
 # Author: M.M. (Rene) van Paassen (using xferfcn.py as basis)
 # Date: 02 Oct 12
 
-from __future__ import division
 
 """
 Frequency response data representation and functions.
@@ -48,33 +47,66 @@ FRD data.
 from warnings import warn
 import numpy as np
 from numpy import angle, array, empty, ones, \
-    real, imag, absolute, eye, linalg, where, dot
+    real, imag, absolute, eye, linalg, where, sort
 from scipy.interpolate import splprep, splev
-from .lti import LTI
+from .lti import LTI, _process_frequency_response
+from . import config
 
 __all__ = ['FrequencyResponseData', 'FRD', 'frd']
 
 
 class FrequencyResponseData(LTI):
-    """FrequencyResponseData(d, w)
+    """FrequencyResponseData(d, w[, smooth])
 
-    A class for models defined by frequency response data (FRD)
+    A class for models defined by frequency response data (FRD).
 
     The FrequencyResponseData (FRD) class is used to represent systems in
     frequency response data form.
 
-    The main data members are 'omega' and 'fresp', where `omega` is a 1D array
-    with the frequency points of the response, and `fresp` is a 3D array, with
-    the first dimension corresponding to the output index of the FRD, the
+    Parameters
+    ----------
+    d : 1D or 3D complex array_like
+        The frequency response at each frequency point.  If 1D, the system is
+        assumed to be SISO.  If 3D, the system is MIMO, with the first
+        dimension corresponding to the output index of the FRD, the second
+        dimension corresponding to the input index, and the 3rd dimension
+        corresponding to the frequency points in omega
+    w : iterable of real frequencies
+        List of frequency points for which data are available.
+    smooth : bool, optional
+        If ``True``, create an interpolation function that allows the
+        frequency response to be computed at any frequency within the range of
+        frequencies give in ``w``.  If ``False`` (default), frequency response
+        can only be obtained at the frequencies specified in ``w``.
+
+    Attributes
+    ----------
+    ninputs, noutputs : int
+        Number of input and output variables.
+    omega : 1D array
+        Frequency points of the response.
+    fresp : 3D array
+        Frequency response, indexed by output index, input index, and
+        frequency point.
+
+    Notes
+    -----
+    The main data members are 'omega' and 'fresp', where 'omega' is a 1D array
+    of frequency points and and 'fresp' is a 3D array of frequency responses,
+    with the first dimension corresponding to the output index of the FRD, the
     second dimension corresponding to the input index, and the 3rd dimension
     corresponding to the frequency points in omega.  For example,
 
     >>> frdata[2,5,:] = numpy.array([1., 0.8-0.2j, 0.2-0.8j])
 
-    means that the frequency response from the 6th input to the 3rd
-    output at the frequencies defined in omega is set to the array
-    above, i.e. the rows represent the outputs and the columns
-    represent the inputs.
+    means that the frequency response from the 6th input to the 3rd output at
+    the frequencies defined in omega is set to the array above, i.e. the rows
+    represent the outputs and the columns represent the inputs.
+
+    A frequency response data object is callable and returns the value of the
+    transfer function evaluated at a point in the complex plane (must be on
+    the imaginary access).  See :meth:`~control.FrequencyResponseData.__call__`
+    for a more detailed description.
 
     """
 
@@ -82,7 +114,24 @@ class FrequencyResponseData(LTI):
     # https://docs.scipy.org/doc/numpy/reference/arrays.classes.html
     __array_priority__ = 11     # override ndarray and matrix types
 
-    epsw = 1e-8
+    #
+    # Class attributes
+    #
+    # These attributes are defined as class attributes so that they are
+    # documented properly.  They are "overwritten" in __init__.
+    #
+
+    #: Number of system inputs.
+    #:
+    #: :meta hide-value:
+    ninputs = 1
+
+    #: Number of system outputs.
+    #:
+    #: :meta hide-value:
+    noutputs = 1
+
+    _epsw = 1e-8                #: Bound for exact frequency match
 
     def __init__(self, *args, **kwargs):
         """Construct an FRD object.
@@ -100,6 +149,7 @@ class FrequencyResponseData(LTI):
         object, other than an FRD, call FRD(sys, omega)
 
         """
+        # TODO: discrete-time FRD systems?
         smooth = kwargs.get('smooth', False)
 
         if len(args) == 2:
@@ -107,16 +157,14 @@ class FrequencyResponseData(LTI):
                 # not an FRD, but still a system, second argument should be
                 # the frequency range
                 otherlti = args[0]
-                self.omega = array(args[1], dtype=float)
-                self.omega.sort()
-                numfreq = len(self.omega)
-
+                self.omega = sort(np.asarray(args[1], dtype=float))
                 # calculate frequency response at my points
-                self.fresp = empty(
-                    (otherlti.outputs, otherlti.inputs, numfreq),
-                    dtype=complex)
-                for k, w in enumerate(self.omega):
-                    self.fresp[:, :, k] = otherlti._evalfr(w)
+                if otherlti.isctime():
+                    s = 1j * self.omega
+                    self.fresp = otherlti(s, squeeze=False)
+                else:
+                    z = np.exp(1j * self.omega * otherlti.dt)
+                    self.fresp = otherlti(z, squeeze=False)
 
             else:
                 # The user provided a response and a freq vector
@@ -141,7 +189,8 @@ class FrequencyResponseData(LTI):
             self.omega = args[0].omega
             self.fresp = args[0].fresp
         else:
-            raise ValueError("Needs 1 or 2 arguments; receivd %i." % len(args))
+            raise ValueError(
+                "Needs 1 or 2 arguments; received %i." % len(args))
 
         # create interpolation functions
         if smooth:
@@ -160,23 +209,32 @@ class FrequencyResponseData(LTI):
     def __str__(self):
         """String representation of the transfer function."""
 
-        mimo = self.inputs > 1 or self.outputs > 1
-        outstr = ['frequency response data ']
+        mimo = self.ninputs > 1 or self.noutputs > 1
+        outstr = ['Frequency response data']
 
-        mt, pt, wt = self.freqresp(self.omega)
-        for i in range(self.inputs):
-            for j in range(self.outputs):
+        for i in range(self.ninputs):
+            for j in range(self.noutputs):
                 if mimo:
                     outstr.append("Input %i to output %i:" % (i + 1, j + 1))
-                outstr.append('Freq [rad/s]  Response   ')
+                outstr.append('Freq [rad/s]  Response')
                 outstr.append('------------  ---------------------')
                 outstr.extend(
-                    ['%12.3f  %10.4g%+10.4gj' % (w, m, p)
-                     for m, p, w in zip(real(self.fresp[j, i, :]),
-                                        imag(self.fresp[j, i, :]), wt)])
+                    ['%12.3f  %10.4g%+10.4gj' % (w, re, im)
+                     for w, re, im in zip(self.omega,
+                                          real(self.fresp[j, i, :]),
+                                          imag(self.fresp[j, i, :]))])
 
         return '\n'.join(outstr)
 
+    def __repr__(self):
+        """Loadable string representation,
+
+        limited for number of data points.
+        """
+        return "FrequencyResponseData({d}, {w}{smooth})".format(
+            d=repr(self.fresp), w=repr(self.omega),
+            smooth=(self.ifunc and ", smooth=True") or "")
+
     def __neg__(self):
         """Negate a transfer function."""
 
@@ -194,15 +252,15 @@ class FrequencyResponseData(LTI):
 
         # Convert the second argument to a frequency response function.
         # or re-base the frd to the current omega (if needed)
-        other = _convertToFRD(other, omega=self.omega)
+        other = _convert_to_FRD(other, omega=self.omega)
 
         # Check that the input-output sizes are consistent.
-        if self.inputs != other.inputs:
+        if self.ninputs != other.ninputs:
             raise ValueError("The first summand has %i input(s), but the \
-second has %i." % (self.inputs, other.inputs))
-        if self.outputs != other.outputs:
+second has %i." % (self.ninputs, other.ninputs))
+        if self.noutputs != other.noutputs:
             raise ValueError("The first summand has %i output(s), but the \
-second has %i." % (self.outputs, other.outputs))
+second has %i." % (self.noutputs, other.noutputs))
 
         return FRD(self.fresp + other.fresp, other.omega)
 
@@ -229,21 +287,21 @@ second has %i." % (self.outputs, other.outputs))
             return FRD(self.fresp * other, self.omega,
                        smooth=(self.ifunc is not None))
         else:
-            other = _convertToFRD(other, omega=self.omega)
+            other = _convert_to_FRD(other, omega=self.omega)
 
         # Check that the input-output sizes are consistent.
-        if self.inputs != other.outputs:
+        if self.ninputs != other.noutputs:
             raise ValueError(
                 "H = G1*G2: input-output size mismatch: "
                 "G1 has %i input(s), G2 has %i output(s)." %
-                (self.inputs, other.outputs))
+                (self.ninputs, other.noutputs))
 
-        inputs = other.inputs
-        outputs = self.outputs
+        inputs = other.ninputs
+        outputs = self.noutputs
         fresp = empty((outputs, inputs, len(self.omega)),
                       dtype=self.fresp.dtype)
         for i in range(len(self.omega)):
-            fresp[:, :, i] = dot(self.fresp[:, :, i], other.fresp[:, :, i])
+            fresp[:, :, i] = self.fresp[:, :, i] @ other.fresp[:, :, i]
         return FRD(fresp, self.omega,
                    smooth=(self.ifunc is not None) and
                           (other.ifunc is not None))
@@ -256,22 +314,22 @@ second has %i." % (self.outputs, other.outputs))
             return FRD(self.fresp * other, self.omega,
                        smooth=(self.ifunc is not None))
         else:
-            other = _convertToFRD(other, omega=self.omega)
+            other = _convert_to_FRD(other, omega=self.omega)
 
         # Check that the input-output sizes are consistent.
-        if self.outputs != other.inputs:
+        if self.noutputs != other.ninputs:
             raise ValueError(
                 "H = G1*G2: input-output size mismatch: "
                 "G1 has %i input(s), G2 has %i output(s)." %
-                (other.inputs, self.outputs))
+                (other.ninputs, self.noutputs))
 
-        inputs = self.inputs
-        outputs = other.outputs
+        inputs = self.ninputs
+        outputs = other.noutputs
 
         fresp = empty((outputs, inputs, len(self.omega)),
                       dtype=self.fresp.dtype)
         for i in range(len(self.omega)):
-            fresp[:, :, i] = dot(other.fresp[:, :, i], self.fresp[:, :, i])
+            fresp[:, :, i] = other.fresp[:, :, i] @ self.fresp[:, :, i]
         return FRD(fresp, self.omega,
                    smooth=(self.ifunc is not None) and
                           (other.ifunc is not None))
@@ -284,10 +342,10 @@ second has %i." % (self.outputs, other.outputs))
             return FRD(self.fresp * (1/other), self.omega,
                        smooth=(self.ifunc is not None))
         else:
-            other = _convertToFRD(other, omega=self.omega)
+            other = _convert_to_FRD(other, omega=self.omega)
 
-        if (self.inputs > 1 or self.outputs > 1 or
-            other.inputs > 1 or other.outputs > 1):
+        if (self.ninputs > 1 or self.noutputs > 1 or
+            other.ninputs > 1 or other.noutputs > 1):
             raise NotImplementedError(
                 "FRD.__truediv__ is currently only implemented for SISO "
                 "systems.")
@@ -307,10 +365,10 @@ second has %i." % (self.outputs, other.outputs))
             return FRD(other / self.fresp, self.omega,
                        smooth=(self.ifunc is not None))
         else:
-            other = _convertToFRD(other, omega=self.omega)
+            other = _convert_to_FRD(other, omega=self.omega)
 
-        if (self.inputs > 1 or self.outputs > 1 or
-            other.inputs > 1 or other.outputs > 1):
+        if (self.ninputs > 1 or self.noutputs > 1 or
+            other.ninputs > 1 or other.noutputs > 1):
             raise NotImplementedError(
                 "FRD.__rtruediv__ is currently only implemented for "
                 "SISO systems.")
@@ -333,120 +391,159 @@ second has %i." % (self.outputs, other.outputs))
             return (FRD(ones(self.fresp.shape), self.omega) / self) * \
                 (self**(other+1))
 
-    def evalfr(self, omega):
-        """Evaluate a transfer function at a single angular frequency.
-
-        self._evalfr(omega) returns the value of the frequency response
-        at frequency omega.
-
-        Note that a "normal" FRD only returns values for which there is an
-        entry in the omega vector. An interpolating FRD can return
-        intermediate values.
-
-        """
-        warn("FRD.evalfr(omega) will be deprecated in a future release "
-             "of python-control; use sys.eval(omega) instead",
-             PendingDeprecationWarning)         # pragma: no coverage
-        return self._evalfr(omega)
-
     # Define the `eval` function to evaluate an FRD at a given (real)
     # frequency.  Note that we choose to use `eval` instead of `evalfr` to
     # avoid confusion with :func:`evalfr`, which takes a complex number as its
     # argument.  Similarly, we don't use `__call__` to avoid confusion between
     # G(s) for a transfer function and G(omega) for an FRD object.
-    def eval(self, omega):
-        """Evaluate a transfer function at a single angular frequency.
-
-        self.evalfr(omega) returns the value of the frequency response
-        at frequency omega.
+    # update Sawyer B. Fuller 2020.08.14: __call__ added to provide a uniform
+    # interface to systems in general and the lti.frequency_response method
+    def eval(self, omega, squeeze=None):
+        """Evaluate a transfer function at angular frequency omega.
 
         Note that a "normal" FRD only returns values for which there is an
         entry in the omega vector. An interpolating FRD can return
         intermediate values.
 
-        """
-        return self._evalfr(omega)
+        Parameters
+        ----------
+        omega : float or 1D array_like
+            Frequencies in radians per second
+        squeeze : bool, optional
+            If squeeze=True, remove single-dimensional entries from the shape
+            of the output even if the system is not SISO. If squeeze=False,
+            keep all indices (output, input and, if omega is array_like,
+            frequency) even if the system is SISO. The default value can be
+            set using config.defaults['control.squeeze_frequency_response'].
 
-    # Internal function to evaluate the frequency responses
-    def _evalfr(self, omega):
-        """Evaluate a transfer function at a single angular frequency."""
-        # Preallocate the output.
-        if getattr(omega, '__iter__', False):
-            out = empty((self.outputs, self.inputs, len(omega)), dtype=complex)
-        else:
-            out = empty((self.outputs, self.inputs), dtype=complex)
+        Returns
+        -------
+        fresp : complex ndarray
+            The frequency response of the system.  If the system is SISO and
+            squeeze is not True, the shape of the array matches the shape of
+            omega.  If the system is not SISO or squeeze is False, the first
+            two dimensions of the array are indices for the output and input
+            and the remaining dimensions match omega.  If ``squeeze`` is True
+            then single-dimensional axes are removed.
+
+        """
+        omega_array = np.array(omega, ndmin=1)  # array-like version of omega
+
+        # Make sure that we are operating on a simple list
+        if len(omega_array.shape) > 1:
+            raise ValueError("input list must be 1D")
+
+        # Make sure that frequencies are all real-valued
+        if any(omega_array.imag > 0):
+            raise ValueError("FRD.eval can only accept real-valued omega")
 
         if self.ifunc is None:
-            try:
-                out = self.fresp[:, :, where(self.omega == omega)[0][0]]
-            except Exception:
+            elements = np.isin(self.omega, omega)  # binary array
+            if sum(elements) < len(omega_array):
                 raise ValueError(
-                    "Frequency %f not in frequency list, try an interpolating"
-                    " FRD if you want additional points" % omega)
-        else:
-            if getattr(omega, '__iter__', False):
-                for i in range(self.outputs):
-                    for j in range(self.inputs):
-                        for k, w in enumerate(omega):
-                            frraw = splev(w, self.ifunc[i, j], der=0)
-                            out[i, j, k] = frraw[0] + 1.0j * frraw[1]
+                    "not all frequencies omega are in frequency list of FRD "
+                    "system. Try an interpolating FRD for additional points.")
             else:
-                for i in range(self.outputs):
-                    for j in range(self.inputs):
-                        frraw = splev(omega, self.ifunc[i, j], der=0)
-                        out[i, j] = frraw[0] + 1.0j * frraw[1]
+                out = self.fresp[:, :, elements]
+        else:
+            out = empty((self.noutputs, self.ninputs, len(omega_array)),
+                        dtype=complex)
+            for i in range(self.noutputs):
+                for j in range(self.ninputs):
+                    for k, w in enumerate(omega_array):
+                        frraw = splev(w, self.ifunc[i, j], der=0)
+                        out[i, j, k] = frraw[0] + 1.0j * frraw[1]
 
-        return out
+        return _process_frequency_response(self, omega, out, squeeze=squeeze)
 
-    # Method for generating the frequency response of the system
-    def freqresp(self, omega):
-        """Evaluate a transfer function at a list of angular frequencies.
+    def __call__(self, s, squeeze=None):
+        """Evaluate system's transfer function at complex frequencies.
 
-        mag, phase, omega = self.freqresp(omega)
+        Returns the complex frequency response `sys(s)` of system `sys` with
+        `m = sys.ninputs` number of inputs and `p = sys.noutputs` number of
+        outputs.
 
-        reports the value of the magnitude, phase, and angular frequency of
-        the transfer function matrix evaluated at s = i * omega, where omega
-        is a list of angular frequencies, and is a sorted version of the input
-        omega.
+        To evaluate at a frequency omega in radians per second, enter
+        ``s = omega * 1j`` or use ``sys.eval(omega)``
+
+        For a frequency response data object, the argument must be an
+        imaginary number (since only the frequency response is defined).
+
+        Parameters
+        ----------
+        s : complex scalar or 1D array_like
+            Complex frequencies
+        squeeze : bool, optional (default=True)
+            If squeeze=True, remove single-dimensional entries from the shape
+            of the output even if the system is not SISO. If squeeze=False,
+            keep all indices (output, input and, if omega is array_like,
+            frequency) even if the system is SISO. The default value can be
+            set using config.defaults['control.squeeze_frequency_response'].
+
+        Returns
+        -------
+        fresp : complex ndarray
+            The frequency response of the system.  If the system is SISO and
+            squeeze is not True, the shape of the array matches the shape of
+            omega.  If the system is not SISO or squeeze is False, the first
+            two dimensions of the array are indices for the output and input
+            and the remaining dimensions match omega.  If ``squeeze`` is True
+            then single-dimensional axes are removed.
+
+        Raises
+        ------
+        ValueError
+            If `s` is not purely imaginary, because
+            :class:`FrequencyDomainData` systems are only defined at imaginary
+            frequency values.
 
         """
+        # Make sure that we are operating on a simple list
+        if len(np.atleast_1d(s).shape) > 1:
+            raise ValueError("input list must be 1D")
 
-        # Preallocate outputs.
-        numfreq = len(omega)
-        mag = empty((self.outputs, self.inputs, numfreq))
-        phase = empty((self.outputs, self.inputs, numfreq))
+        if any(abs(np.atleast_1d(s).real) > 0):
+            raise ValueError("__call__: FRD systems can only accept "
+                             "purely imaginary frequencies")
 
-        omega.sort()
+        # need to preserve array or scalar status
+        if hasattr(s, '__len__'):
+            return self.eval(np.asarray(s).imag, squeeze=squeeze)
+        else:
+            return self.eval(complex(s).imag, squeeze=squeeze)
 
-        for k, w in enumerate(omega):
-            fresp = self._evalfr(w)
-            mag[:, :, k] = abs(fresp)
-            phase[:, :, k] = angle(fresp)
+    def freqresp(self, omega):
+        """(deprecated) Evaluate transfer function at complex frequencies.
 
-        return mag, phase, omega
+        .. deprecated::0.9.0
+            Method has been given the more pythonic name
+            :meth:`FrequencyResponseData.frequency_response`. Or use
+            :func:`freqresp` in the MATLAB compatibility module.
+        """
+        warn("FrequencyResponseData.freqresp(omega) will be removed in a "
+             "future release of python-control; use "
+             "FrequencyResponseData.frequency_response(omega), or "
+             "freqresp(sys, omega) in the MATLAB compatibility module "
+             "instead", DeprecationWarning)
+        return self.frequency_response(omega)
 
     def feedback(self, other=1, sign=-1):
         """Feedback interconnection between two FRD objects."""
 
-        other = _convertToFRD(other, omega=self.omega)
+        other = _convert_to_FRD(other, omega=self.omega)
 
-        if (self.outputs != other.inputs or self.inputs != other.outputs):
+        if (self.noutputs != other.ninputs or self.ninputs != other.noutputs):
             raise ValueError(
                 "FRD.feedback, inputs/outputs mismatch")
-        fresp = empty((self.outputs, self.inputs, len(other.omega)),
-                      dtype=complex)
-        # TODO: vectorize this
+
         # TODO: handle omega re-mapping
-        # TODO: is there a reason to use linalg.solve instead of linalg.inv?
-        # https://github.com/python-control/python-control/pull/314#discussion_r294075154
-        for k, w in enumerate(other.omega):
-            fresp[:, :, k] = np.dot(
-                self.fresp[:, :, k],
-                linalg.solve(
-                    eye(self.inputs)
-                    + np.dot(other.fresp[:, :, k], self.fresp[:, :, k]),
-                    eye(self.inputs))
-            )
+
+        # reorder array axes in order to leverage numpy broadcasting
+        myfresp = np.moveaxis(self.fresp, 2, 0)
+        otherfresp = np.moveaxis(other.fresp, 2, 0)
+        I_AB = eye(self.ninputs)[np.newaxis, :, :] + otherfresp @ myfresp
+        resfresp = (myfresp @ linalg.inv(I_AB))
+        fresp = np.moveaxis(resfresp, 0, 2)
 
         return FRD(fresp, other.omega, smooth=(self.ifunc is not None))
 
@@ -461,10 +558,11 @@ second has %i." % (self.outputs, other.outputs))
 # fixes this problem.
 #
 
+
 FRD = FrequencyResponseData
 
 
-def _convertToFRD(sys, omega, inputs=1, outputs=1):
+def _convert_to_FRD(sys, omega, inputs=1, outputs=1):
     """Convert a system to frequency response data form (if needed).
 
     If sys is already an frd, and its frequency range matches or
@@ -474,8 +572,8 @@ def _convertToFRD(sys, omega, inputs=1, outputs=1):
     scalar, then the number of inputs and outputs can be specified
     manually, as in:
 
-    >>> frd = _convertToFRD(3., omega) # Assumes inputs = outputs = 1
-    >>> frd = _convertToFRD(1., omegs, inputs=3, outputs=2)
+    >>> frd = _convert_to_FRD(3., omega) # Assumes inputs = outputs = 1
+    >>> frd = _convert_to_FRD(1., omegs, inputs=3, outputs=2)
 
     In the latter example, sys's matrix transfer function is [[1., 1., 1.]
                                                               [1., 1., 1.]].
@@ -485,7 +583,7 @@ def _convertToFRD(sys, omega, inputs=1, outputs=1):
     if isinstance(sys, FRD):
         omega.sort()
         if len(omega) == len(sys.omega) and \
-           (abs(omega - sys.omega) < FRD.epsw).all():
+           (abs(omega - sys.omega) < FRD._epsw).all():
             # frequencies match, and system was already frd; simply use
             return sys
 
@@ -493,11 +591,13 @@ def _convertToFRD(sys, omega, inputs=1, outputs=1):
             "Frequency ranges of FRD do not match, conversion not implemented")
 
     elif isinstance(sys, LTI):
-        omega.sort()
-        fresp = empty((sys.outputs, sys.inputs, len(omega)), dtype=complex)
-        for k, w in enumerate(omega):
-            fresp[:, :, k] = sys._evalfr(w)
-
+        omega = np.sort(omega)
+        if sys.isctime():
+            fresp = sys(1j * omega)
+        else:
+            fresp = sys(np.exp(1j * omega * sys.dt))
+        if len(fresp.shape) == 1:
+            fresp = fresp[np.newaxis, np.newaxis, :]
         return FRD(fresp, omega, smooth=True)
 
     elif isinstance(sys, (int, float, complex, np.number)):
