@@ -70,7 +70,7 @@ from ..exception import ControlArgument
 
 # Import MATLAB-like functions that can be used as-is
 from ..ctrlutil import *
-from ..freqplot import nyquist, gangof4
+from ..freqplot import gangof4
 from ..nichols import nichols
 from ..bdalg import *
 from ..pzmap import *
@@ -81,11 +81,16 @@ from ..mateqn import *
 from ..margins import margin
 from ..rlocus import rlocus
 from ..dtime import c2d
+from ..sisotool import sisotool
 
 # Import functions specific to Matlab compatibility package
 from .timeresp import *
 from .wrappers import *
 
+# Set up defaults corresponding to MATLAB conventions
+from ..config import *
+use_matlab_defaults()
+
 r"""
 The following tables give an overview of the module ``control.matlab``.
 They also show the implementation progress and the planned features of the
@@ -219,8 +224,8 @@ Frequency-domain analysis
 \*  :func:`~control.nichols`    Nichols plot
 \*  :func:`margin`              gain and phase margins
 \   lti/allmargin               all crossover frequencies and margins
-\*  :func:`freqresp`            frequency response over a frequency grid
-\*  :func:`evalfr`              frequency response at single frequency
+\*  :func:`freqresp`            frequency response
+\*  :func:`evalfr`              frequency response at complex frequency s
 ==  ==========================  ============================================
 
 
@@ -241,6 +246,7 @@ Compensator design
 
 ==  ==========================  ============================================
 \*  :func:`rlocus`              evans root locus
+\*  :func:`sisotool`            SISO controller design
 \*  :func:`~control.place`      pole placement
 \   estim                       form estimator given estimator gain
 \   reg                         form regulator given state-feedback and
